@@ -1,18 +1,15 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
 
-import warnings
-import itertools
 import functools
+import itertools
+import warnings
 
 import numpy as np
-
-from ..core.pycompat import getargspec
 from ..core.formatting import format_item
-from .utils import (_determine_cmap_params, _infer_xy_labels,
-                    import_matplotlib_pyplot)
-
+from ..core.pycompat import getargspec
+from .utils import (
+    _determine_cmap_params, _infer_xy_labels, import_matplotlib_pyplot,
+    label_from_attrs)
 
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
 # from mpl.rcParams
@@ -221,6 +218,14 @@ class FacetGrid(object):
         self : FacetGrid object
 
         """
+
+        cmapkw = kwargs.get('cmap')
+        colorskw = kwargs.get('colors')
+
+        # colors is mutually exclusive with cmap
+        if cmapkw and colorskw:
+            raise ValueError("Can't specify both cmap and colors.")
+
         # These should be consistent with xarray.plot._plot2d
         cmap_kwargs = {'plot_data': self.data.values,
                        # MPL default
@@ -233,6 +238,9 @@ class FacetGrid(object):
 
         cmap_params = _determine_cmap_params(**cmap_kwargs)
 
+        if colorskw is not None:
+            cmap_params['cmap'] = None
+
         # Order is important
         func_kwargs = kwargs.copy()
         func_kwargs.update(cmap_params)
@@ -258,6 +266,46 @@ class FacetGrid(object):
 
         return self
 
+    def map_dataarray_line(self, x=None, y=None, hue=None, **kwargs):
+        """
+        Apply a line plot to a 2d facet subset of the data.
+
+        Parameters
+        ----------
+        x, y, hue: string
+            dimension names for the axes and hues of each facet
+
+        Returns
+        -------
+        self : FacetGrid object
+
+        """
+        from .plot import line, _infer_line_data
+
+        add_legend = kwargs.pop('add_legend', True)
+        kwargs['add_legend'] = False
+
+        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+            # None is the sentinel value
+            if d is not None:
+                subset = self.data.loc[d]
+                mappable = line(subset, x=x, y=y, hue=hue,
+                                ax=ax, _labels=False,
+                                **kwargs)
+                self._mappables.append(mappable)
+        _, _, hueplt, xlabel, ylabel, huelabel = _infer_line_data(
+            darray=self.data.loc[self.name_dicts.flat[0]],
+            x=x, y=y, hue=hue)
+
+        self._hue_var = hueplt
+        self._hue_label = huelabel
+        self._finalize_grid(xlabel, ylabel)
+
+        if add_legend and hueplt is not None and huelabel is not None:
+            self.add_legend()
+
+        return self
+
     def _finalize_grid(self, *axlabels):
         """Finalize the annotations and layout."""
         self.set_axis_labels(*axlabels)
@@ -268,14 +316,42 @@ class FacetGrid(object):
             if namedict is None:
                 ax.set_visible(False)
 
+    def add_legend(self, **kwargs):
+        figlegend = self.fig.legend(
+            handles=self._mappables[-1],
+            labels=list(self._hue_var.values),
+            title=self._hue_label,
+            loc="center right", **kwargs)
+
+        # Draw the plot to set the bounding boxes correctly
+        self.fig.draw(self.fig.canvas.get_renderer())
+
+        # Calculate and set the new width of the figure so the legend fits
+        legend_width = figlegend.get_window_extent().width / self.fig.dpi
+        figure_width = self.fig.get_figwidth()
+        self.fig.set_figwidth(figure_width + legend_width)
+
+        # Draw the plot again to get the new transformations
+        self.fig.draw(self.fig.canvas.get_renderer())
+
+        # Now calculate how much space we need on the right side
+        legend_width = figlegend.get_window_extent().width / self.fig.dpi
+        space_needed = legend_width / (figure_width + legend_width) + 0.02
+        # margin = .01
+        # _space_needed = margin + space_needed
+        right = 1 - space_needed
+
+        # Place the subplot axes to give space for the legend
+        self.fig.subplots_adjust(right=right)
+
     def add_colorbar(self, **kwargs):
         """Draw a colorbar
         """
         kwargs = kwargs.copy()
         if self._cmap_extend is not None:
             kwargs.setdefault('extend', self._cmap_extend)
-        if getattr(self.data, 'name', None) is not None:
-            kwargs.setdefault('label', self.data.name)
+        if 'label' not in kwargs:
+            kwargs.setdefault('label', label_from_attrs(self.data))
         self.cbar = self.fig.colorbar(self._mappables[-1],
                                       ax=list(self.axes.flat),
                                       **kwargs)
@@ -284,17 +360,25 @@ class FacetGrid(object):
     def set_axis_labels(self, x_var=None, y_var=None):
         """Set axis labels on the left column and bottom row of the grid."""
         if x_var is not None:
-            self._x_var = x_var
-            self.set_xlabels(x_var)
+            if x_var in self.data.coords:
+                self._x_var = x_var
+                self.set_xlabels(label_from_attrs(self.data[x_var]))
+            else:
+                # x_var is a string
+                self.set_xlabels(x_var)
+
         if y_var is not None:
-            self._y_var = y_var
-            self.set_ylabels(y_var)
+            if y_var in self.data.coords:
+                self._y_var = y_var
+                self.set_ylabels(label_from_attrs(self.data[y_var]))
+            else:
+                self.set_ylabels(y_var)
         return self
 
     def set_xlabels(self, label=None, **kwargs):
         """Label the x axis on the bottom row of the grid."""
         if label is None:
-            label = self._x_var
+            label = label_from_attrs(self.data[self._x_var])
         for ax in self._bottom_axes:
             ax.set_xlabel(label, **kwargs)
         return self
@@ -302,7 +386,7 @@ class FacetGrid(object):
     def set_ylabels(self, label=None, **kwargs):
         """Label the y axis on the left column of the grid."""
         if label is None:
-            label = self._y_var
+            label = label_from_attrs(self.data[self._y_var])
         for ax in self._left_axes:
             ax.set_ylabel(label, **kwargs)
         return self
