@@ -54,15 +54,13 @@ def _importorskip(modname, minversion=None):
                 raise ImportError('Minimum version not satisfied')
     except ImportError:
         has = False
-    # TODO: use pytest.skipif instead of unittest.skipUnless
-    # Using `unittest.skipUnless` is a temporary workaround for pytest#568,
-    # wherein class decorators stain inherited classes.
-    # xref: xarray#1531, implemented in xarray #1557.
-    func = unittest.skipUnless(has, reason='requires {}'.format(modname))
+    func = pytest.mark.skipif(not has, reason='requires {}'.format(modname))
     return has, func
 
 
 has_matplotlib, requires_matplotlib = _importorskip('matplotlib')
+has_matplotlib2, requires_matplotlib2 = _importorskip('matplotlib',
+                                                      minversion='2')
 has_scipy, requires_scipy = _importorskip('scipy')
 has_pydap, requires_pydap = _importorskip('pydap.client')
 has_netCDF4, requires_netCDF4 = _importorskip('netCDF4')
@@ -75,15 +73,16 @@ has_bottleneck, requires_bottleneck = _importorskip('bottleneck')
 has_rasterio, requires_rasterio = _importorskip('rasterio')
 has_pathlib, requires_pathlib = _importorskip('pathlib')
 has_zarr, requires_zarr = _importorskip('zarr', minversion='2.2')
-has_np112, requires_np112 = _importorskip('numpy', minversion='1.12.0')
+has_np113, requires_np113 = _importorskip('numpy', minversion='1.13.0')
+has_iris, requires_iris = _importorskip('iris')
 
 # some special cases
 has_scipy_or_netCDF4 = has_scipy or has_netCDF4
-requires_scipy_or_netCDF4 = unittest.skipUnless(
-    has_scipy_or_netCDF4, reason='requires scipy or netCDF4')
+requires_scipy_or_netCDF4 = pytest.mark.skipif(
+    not has_scipy_or_netCDF4, reason='requires scipy or netCDF4')
 has_cftime_or_netCDF4 = has_cftime or has_netCDF4
-requires_cftime_or_netCDF4 = unittest.skipUnless(
-    has_cftime_or_netCDF4, reason='requires cftime or netCDF4')
+requires_cftime_or_netCDF4 = pytest.mark.skipif(
+    not has_cftime_or_netCDF4, reason='requires cftime or netCDF4')
 if not has_pathlib:
     has_pathlib, requires_pathlib = _importorskip('pathlib2')
 if has_dask:
@@ -97,7 +96,8 @@ try:
     has_seaborn = True
 except ImportError:
     has_seaborn = False
-requires_seaborn = unittest.skipUnless(has_seaborn, reason='requires seaborn')
+requires_seaborn = pytest.mark.skipif(not has_seaborn,
+                                      reason='requires seaborn')
 
 try:
     _SKIP_FLAKY = not pytest.config.getoption("--run-flaky")
