@@ -10,7 +10,7 @@ del get_versions
 from .core.alignment import align, broadcast, broadcast_arrays
 from .core.common import full_like, zeros_like, ones_like
 from .core.combine import concat, auto_combine
-from .core.computation import apply_ufunc, where, dot
+from .core.computation import apply_ufunc, dot, where
 from .core.extensions import (register_dataarray_accessor,
                               register_dataset_accessor)
 from .core.variable import as_variable, Variable, IndexVariable, Coordinate
@@ -26,6 +26,7 @@ from .backends.zarr import open_zarr
 
 from .conventions import decode_cf, SerializationWarning
 
+from .coding.cftime_offsets import cftime_range
 from .coding.cftimeindex import CFTimeIndex
 
 from .util.print_versions import show_versions
