@@ -12,25 +12,21 @@ from .common import (
     HDF5_LOCK, DataStorePickleMixin, WritableCFDataStore, find_root)
 from .netCDF4_ import (
     BaseNetCDF4Array, _encode_nc4_variable, _extract_nc4_variable_encoding,
-    _get_datatype, _nc4_group)
+    _get_datatype, _nc4_require_group)
 
 
 class H5NetCDFArrayWrapper(BaseNetCDF4Array):
     def __getitem__(self, key):
-        key, np_inds = indexing.decompose_indexer(
-            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR)
+        return indexing.explicit_indexing_adapter(
+            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR,
+            self._getitem)
 
+    def _getitem(self, key):
         # h5py requires using lists for fancy indexing:
         # https://github.com/h5py/h5py/issues/992
-        key = tuple(list(k) if isinstance(k, np.ndarray) else k for k in
-                    key.tuple)
+        key = tuple(list(k) if isinstance(k, np.ndarray) else k for k in key)
         with self.datastore.ensure_open(autoclose=True):
-            array = self.get_array()[key]
-
-        if len(np_inds.tuple) > 0:
-            array = indexing.NumpyIndexingAdapter(array)[np_inds]
-
-        return array
+            return self.get_array()[key]
 
 
 def maybe_decode_bytes(txt):
@@ -57,11 +53,16 @@ _extract_h5nc_encoding = functools.partial(
     lsd_okay=False, h5py_okay=True, backend='h5netcdf')
 
 
+def _h5netcdf_create_group(dataset, name):
+    return dataset.create_group(name)
+
+
 def _open_h5netcdf_group(filename, mode, group):
     import h5netcdf
     ds = h5netcdf.File(filename, mode=mode)
     with close_on_error(ds):
-        return _nc4_group(ds, group, mode)
+        return _nc4_require_group(
+            ds, group, mode, create_group=_h5netcdf_create_group)
 
 
 class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
@@ -89,6 +90,8 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
         super(H5NetCDFStore, self).__init__(writer, lock=lock)
 
     def open_store_variable(self, name, var):
+        import h5py
+
         with self.ensure_open(autoclose=False):
             dimensions = var.dimensions
             data = indexing.LazilyOuterIndexedArray(
@@ -114,6 +117,15 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
             encoding['source'] = self._filename
             encoding['original_shape'] = var.shape
 
+            vlen_dtype = h5py.check_dtype(vlen=var.dtype)
+            if vlen_dtype is unicode_type:
+                encoding['dtype'] = str
+            elif vlen_dtype is not None:  # pragma: no cover
+                # xarray doesn't support writing arbitrary vlen dtypes yet.
+                pass
+            else:
+                encoding['dtype'] = var.dtype
+
         return Variable(dimensions, data, attrs, encoding)
 
     def get_variables(self):
@@ -156,7 +168,8 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
         import h5py
 
         attrs = variable.attrs.copy()
-        dtype = _get_datatype(variable)
+        dtype = _get_datatype(
+            variable, raise_on_invalid_encoding=check_encoding)
 
         fillvalue = attrs.pop('_FillValue', None)
         if dtype is str and fillvalue is not None:
@@ -184,8 +197,9 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
                 raise ValueError("'zlib' and 'compression' encodings mismatch")
             encoding.setdefault('compression', 'gzip')
 
-        if (check_encoding and encoding.get('complevel') not in
-                (None, encoding.get('compression_opts'))):
+        if (check_encoding and
+                'complevel' in encoding and 'compression_opts' in encoding and
+                encoding['complevel'] != encoding['compression_opts']):
             raise ValueError("'complevel' and 'compression_opts' encodings "
                              "mismatch")
         complevel = encoding.pop('complevel', 0)
