@@ -76,13 +76,12 @@ def safe_cast_to_index(array):
 def multiindex_from_product_levels(levels, names=None):
     """Creating a MultiIndex from a product without refactorizing levels.
 
-    Keeping levels the same is faster, and also gives back the original labels
-    when we unstack.
+    Keeping levels the same gives back the original labels when we unstack.
 
     Parameters
     ----------
-    levels : sequence of arrays
-        Unique labels for each level.
+    levels : sequence of pd.Index
+        Values for each MultiIndex level.
     names : optional sequence of objects
         Names for each level.
 
@@ -90,8 +89,11 @@ def multiindex_from_product_levels(levels, names=None):
     -------
     pandas.MultiIndex
     """
-    labels_mesh = np.meshgrid(*[np.arange(len(lev)) for lev in levels],
-                              indexing='ij')
+    if any(not isinstance(lev, pd.Index) for lev in levels):
+        raise TypeError('levels must be a list of pd.Index objects')
+
+    split_labels, levels = zip(*[lev.factorize() for lev in levels])
+    labels_mesh = np.meshgrid(*split_labels, indexing='ij')
     labels = [x.ravel() for x in labels_mesh]
     return pd.MultiIndex(levels, labels, sortorder=0, names=names)
 
@@ -183,7 +185,7 @@ def is_full_slice(value):
     return isinstance(value, slice) and value == slice(None)
 
 
-def combine_pos_and_kw_args(pos_kwargs, kw_kwargs, func_name):
+def either_dict_or_kwargs(pos_kwargs, kw_kwargs, func_name):
     if pos_kwargs is not None:
         if not is_dict_like(pos_kwargs):
             raise ValueError('the first argument to .%s must be a dictionary'
@@ -589,3 +591,24 @@ class HiddenKeyDict(MutableMapping):
     def __len__(self):
         num_hidden = sum([k in self._hidden_keys for k in self._data])
         return len(self._data) - num_hidden
+
+
+def to_numeric(array, offset=None, datetime_unit=None, dtype=float):
+    """
+    Make datetime array float
+
+    offset: Scalar with the same type of array or None
+        If None, subtract minimum values to reduce round off error
+    datetime_unit: None or any of {'Y', 'M', 'W', 'D', 'h', 'm', 's', 'ms',
+        'us', 'ns', 'ps', 'fs', 'as'}
+    dtype: target dtype
+    """
+    if array.dtype.kind not in ['m', 'M']:
+        return array.astype(dtype)
+    if offset is None:
+        offset = np.min(array)
+    array = array - offset
+
+    if datetime_unit:
+        return (array / np.timedelta64(1, datetime_unit)).astype(dtype)
+    return array.astype(dtype)
